#include "ctrl.h"

#include <QMetaType>
#include <QDBusMetaType>
#include <QCoreApplication>
#include <QRegularExpression>
#include <QRegularExpressionMatch>
#include <QDateTime>
#include "usdbrightness.h"

#include <KF5/KScreen/kscreen/output.h>
#include <KF5/KScreen/kscreen/edid.h>
#include <KF5/KScreen/kscreen/mode.h>
#include <KF5/KScreen/kscreen/config.h>
#include <KF5/KScreen/kscreen/getconfigoperation.h>
#include <KF5/KScreen/kscreen/configmonitor.h>
#include <KF5/KScreen/kscreen/setconfigoperation.h>
#include <KF5/KScreen/kscreen/edid.h>
#include <KF5/KScreen/kscreen/types.h>

ctrl::ctrl() : QObject()
{
    if (QGSettings::isSchemaInstalled(QByteArray("org.ukui.control-center.panel.plugins"))) {
        QGSettings gsettings(QByteArray("org.ukui.control-center.panel.plugins"), QByteArray(), this);
//        gsettings.set("gammaforbrightness", true);
    }
    QObject::connect(new KScreen::GetConfigOperation(), &KScreen::GetConfigOperation::finished,
                     [&](KScreen::ConfigOperation *op) {
                         const KScreen::ConfigPtr config = qobject_cast<KScreen::GetConfigOperation *>(op)->config();
                         KScreen::ConfigMonitor::instance()->addConfig(config);
                         for (KScreen::OutputPtr output : config.data()->connectedOutputs()) {
                             qInfo() << "Starting config screen name: " << output->name();
                             m_screenNameList.append(output->name());
                         }
                     });
}

void ctrl::controlMprisPlayerDbus(const QString funcName)
{
    if (m_dbusMprisInterface == nullptr) {
        m_dbusMprisInterface = new QDBusInterface(SESSION_DBUS_NAME_STR_FIELD, SESSION_DBUS_PLAYER_PATH,
                                                  SESSION_DBUS_PLAYER_INTERFACE, QDBusConnection::sessionBus());
    }
    if (m_dbusMprisInterface->isValid()) {
        if (funcName.startsWith("seek")) {
            QStringList itemList = funcName.trimmed().split(':');
            bool ok;
            qlonglong value = itemList.value(1).toLongLong(&ok);
            if (ok) {
                m_dbusMprisInterface->asyncCall("operation", itemList.value(0), QVariantList() << value);
            }
            return;
        }
        if (funcName.startsWith("setRate")) {
            QStringList itemList = funcName.trimmed().split(':');
            bool ok;
            double value = itemList.value(1).toDouble(&ok);
            if (ok) {
                m_dbusMprisInterface->asyncCall("operation", itemList.value(0), QVariantList() << value);
            }
            return;
        }
        m_dbusMprisInterface->asyncCall("operation", funcName, QVariantList());
    }
}

void ctrl::setBrightness(bool isUp)
{
    if (m_dbusBrightnessInterface == nullptr) {
        m_dbusBrightnessInterface = new QDBusInterface(SESSION_DBUS_NAME_BRIGHTNESS, SESSION_DBUS_BRIGHTNESS_PATH,
                                                       SESSION_DBUS_BRIGHTNESS_INTERFACE, QDBusConnection::sessionBus());
    }
    QDBusMessage res =m_dbusBrightnessInterface->call("getPrimaryBrightness");
    int value = res.arguments().first().toUInt();
    qInfo() << "get screen brightness: " << value;
    if (isUp) {
        if (value >= 100) {
            value = 100;
        } else {
            value += 10;
        }
    } else {
        if (value <= 0) {
            value = 0;
        } else {
            value -= 10;
        }
    }
    qInfo() << "set screen brightness: " << value;
    res = m_dbusBrightnessInterface->call("setPrimaryBrightness", (quint32)value);
}

void ctrl::setDefaultAdapterSwitchStatus(bool power)
{
    QDBusMessage dbusMsg = QDBusMessage::createMethodCall(SYSTEM_DBUS_BLE_SERVICE, SYSTEM_DBUS_BLE_PATH,
                                                          SYSTEM_DBUS_BLE_INTERFACE, "setDefaultAdapterAttr");
    QMap<QString, QVariant> adpAttr;
    adpAttr.insert("Powered", QVariant(power));
    dbusMsg << adpAttr;
    QDBusMessage response = QDBusConnection::systemBus().call(dbusMsg, QDBus::NoBlock);

    if (response.type() == QDBusMessage::ReplyMessage) {
        qInfo() << "set Ble status:" << response.arguments().takeFirst().toBool();
    }
}

bool ctrl::addToSchedule(QString summary, QString msg)
{
    qDebug() << summary;
    if (m_scheduleInterface == nullptr) {
        m_scheduleInterface = new QDBusInterface(SESSION_DBUS_NAME_CALENDAR, SESSION_DBUS_PATH_CALENDAR,
                                                  SESSION_DBUS_INTERFACE_CALENDAR, QDBusConnection::sessionBus());
    }

    QString startTimeExtract = extractTime(summary, "开始时间");
    QString endTimeExtract = extractTime(summary, "结束时间");

    qDebug() << "Start Time Extract:" << startTimeExtract;
    qDebug() << "End Time Extract:" << endTimeExtract;


    if (startTimeExtract == "") {
        qDebug() << "未找到开始时间信息";
        return false;
    }

    QDateTime start_time = QDateTime::fromString(startTimeExtract, "yyyy-MM-dd HH:mm:ss");

    if (!start_time.isValid()) {
        qDebug() << "无效的开始时间格式：" << startTimeExtract;
        return false;
    }

    int start_year = start_time.date().year();
    int start_month = start_time.date().month();
    int start_day = start_time.date().day();
    int start_hour = start_time.time().hour();
    int start_minute = start_time.time().minute();


    qDebug() << start_year << "年" << start_month << "月" << start_day << "日" << start_hour << "时" << start_minute << "分";

    QDateTime end_time;
    if (endTimeExtract != "") {
        end_time = QDateTime::fromString(endTimeExtract, "yyyy-MM-dd HH:mm:ss");
        if (!end_time.isValid()) {
            qDebug() << "无效的结束时间格式：" << endTimeExtract;
            end_time = start_time;
        }
    } else {
        qDebug() << "未找到结束时间信息";
        end_time = start_time;
    }

    int end_month = end_time.date().month();
    int end_day = end_time.date().day();
    int end_hour = end_time.time().hour();
    int end_minute = end_time.time().minute();

    qDebug() << end_month << "月" << end_day << "日" << end_hour << "时" << end_minute << "分";

//    QRegularExpression regex_topic("主题：(.*)");
//    QRegularExpressionMatch match_str = regex_topic.match(summary);

//    QString topic;

//    if (match_str.hasMatch()) {
//            QString match_topic = match_str.captured(1);
//            topic = match_topic.mid(0,match_topic.indexOf('\\'));
//            topic.replace("\r", "");
//    } else {
//        qDebug() << "未找到主题信息";
//        topic = "预定的会议";  // 默认的会议主题
//    }

    // 调用DBus接口发送日程安排
    QDBusMessage res = m_scheduleInterface->call("ScheduleNotification",
        start_hour, start_minute, start_day, start_month, start_year,
        end_minute, end_hour, msg);

    if (res.type() == QDBusMessage::ErrorMessage) {
        // 处理错误
        qDebug() << "Error Calling ScheduleNotification:" << res.errorMessage();
        return false;
    } else {
        qDebug() << "Add ScheduleNotification Success!";
        return true;
    }
}

QString ctrl::extractTime(const QString &str, const QString &keyword) {
    QRegularExpression re(QString("(%1：\\s*)(\\d{2,4}-\\d{2}-\\d{2}\\s+\\d{2}:\\d{2}:\\d{2})").arg(keyword));
    QRegularExpressionMatch match = re.match(str);
    if (match.hasMatch()) {
        return match.captured(2);
    }
    return QString(); // 如果没有匹配到，返回空字符串
}
