#include "kaglobal.h"

#include <QStyleOption>
#include <QStyle>
#include <QBitmap>
#include <QPalette>
#include <QPainter>
#include <QDebug>
#include <QMap>
#include <QJsonObject>
#include <QJsonDocument>

bool isSettingOpen = false;
bool isNlpInUse = false;                // nlp-判断正在使用
bool isVisionInUse = false;             // vision-判断正在使用
bool isSpeechInUse = false;             // speech-判断正在使用
bool isReatimeSpeechInUse = false;      // 实时-判断正在使用
bool isContinuousSpeechInUse = false;   // 流式-判断正在使用
bool isContinuousReadInUse = false;     // 朗读-判断正在使用
bool isSpeechSynthesizing = false;      // 是否正在合成语音

int globals::themeColor = 0; // 0:为浅色 1:为深色 2:自由模式
int globals::fontSize = 11;  // 系统默认字体大小为11
QUrl globals::currentPageUrl = QUrl("qrc:/html/html/index1.html");  // 默认主页
QUrl globals::openLink = QUrl(QString("https://docs.openkylin.top/zh/07_%E5%85%B3%E4%BA%8E%E7%A4%BE%E5%8C%BA/"
                                      "%E7%A4%BE%E5%8C%BA%E5%B9%B3%E5%8F%B0%E4%BB%8B%E7%BB%8D/AI%E6%A8%A1%E5%9E%8B%E8%B4%A6%E5%8F%B7%E8%8E%B7%E5%8F%96%E5%8F%8A%E9%85%8D%E7%BD%AE%E6%8C%87%E5%8D%97"));

QUrl globals::useProtocol = QUrl(QString("https://www.openkylin.top/support/ai_user_agreement-cn.html"));


void globals::borderRound(QWidget *widget,int xradius,int yradius){
    if(widget){
        QStyleOption opt;
        opt.initFrom(widget);
        QPainter p(widget);
        widget->style()->drawPrimitive(QStyle::PE_Widget, &opt, &p, widget);//绘制样式

        QBitmap bmp(widget->size());
        bmp.fill();
        QPainter painter(&bmp);
        painter.setPen(Qt::NoPen);
        painter.setBrush(Qt::black);
        painter.setRenderHint(QPainter::Antialiasing);

        //此处可修改圆角矩形大小，也可以绘制为其他想要的形状，比如圆形
        painter.drawRoundedRect(bmp.rect(), xradius, yradius);
        //该函数便使得QBitmap上像素为1的区域才显示出来
        widget->setMask(bmp);
    }
}

void globals::borderColor(QWidget *widget, int xradius, int yradius, QColor borderColor) {
    if (widget) {
        QStyleOption opt;
        opt.initFrom(widget);
        QPainter p(widget);
        widget->style()->drawPrimitive(QStyle::PE_Widget, &opt, &p, widget);//绘制样式

        QBitmap bmp(widget->size());
        bmp.fill();
        QPainter painter(&bmp);
        painter.setPen(Qt::NoPen);
        painter.setBrush(Qt::black);
        painter.setRenderHint(QPainter::Antialiasing);

        //此处可修改圆角矩形大小，也可以绘制为其他想要的形状，比如圆形
        painter.drawRoundedRect(bmp.rect(), xradius, yradius);
        //该函数便使得QBitmap上像素为1的区域才显示出来
        widget->setMask(bmp);
        //绘制外边框
        QPainter borderPainter(widget);
        borderPainter.setRenderHint(QPainter::Antialiasing);
        borderPainter.setPen(QPen(borderColor, 1));
        borderPainter.drawRoundedRect(widget->rect().adjusted(1, 1, -1, -1), xradius, yradius);
    }
}

QString globals::getSettingModelMsg(Capability capability)
{
    DeployPolicy policy = capability_settings_get_deploy_policy(capability);
    int modelLen = capability_settings_get_model_length(capability, policy);
    if(modelLen <= 0)
    {
        return "";
    }
    char model[modelLen + 1];
    QString tmpStr;
    QString str;
    QString keyModel;
    int pos;
    switch(policy)
    {
    case PUBLIC_CLOUD:
        capability_settings_get_model(capability, PUBLIC_CLOUD, model);
        str = QString(model);
        pos = str.indexOf(QChar(0x7F)); // 查找\u007F字符的位置
        if (pos != -1) {
            str = str.left(pos);
        }
        if(str == "baidu"){
            tmpStr = "baidu";
        }else if (str == "xunfei"){
            keyModel = getSettingModelJsonKey(capability, str);
            if (keyModel == "4.0"){
                tmpStr = "xunfei_4";
            }else{
                tmpStr = "xunfei";
            }
        }else if (str == "default"){
            tmpStr = "default";
        }
        break;
    case LOCAL:
        tmpStr = "ondevice";
        break;
    default:
        break;
    }
    return tmpStr;
}


QString globals::getSettingModelJsonKey(Capability capability, const QString &model)
{
    int capabilityModelListLen =
            capability_settings_get_model_list_length(capability, PUBLIC_CLOUD);
    for (int i=0; i<capabilityModelListLen; ++i) {
        const int modelLen = capability_settings_get_model_length_by_index(capability, PUBLIC_CLOUD, i);
        char keyModel[modelLen + 1] = {};
        capability_settings_get_model_by_index(capability, PUBLIC_CLOUD, i, keyModel);
        const int modelConfigLen =
                capability_settings_get_model_config_length(capability, PUBLIC_CLOUD, keyModel);
        char json[modelConfigLen + 1] = {};
        capability_settings_get_model_config(capability, PUBLIC_CLOUD, keyModel, json);
        QJsonObject jsonObj = QJsonDocument::fromJson(json).object();
        if(model == keyModel){
            return jsonObj["model"].toString();
        }
    }
    return "";
}
