#include "kwebengineview.h"

#include <QMimeData>
#include <QDebug>
#include <QFile>
#include <QDataStream>
#include <QFileInfo>

KWebEngineView::KWebEngineView(QWidget *parent)
    : QWebEngineView(parent)
{

}

void KWebEngineView::dragEnterEvent(QDragEnterEvent *event)
{
    emit sigImageDragIn(true);
    //所有图片格式
    QStringList formatList = {"tga", "svg", "gif", "apng", "ico", "xpm",
                              "exr", "psd", "jfi", "jif", "j2k", "jp2", "jng", "wbmp", "xbm", "tiff", "tif", "webp",
                              "pnm", "bmp", "dib", "sr",  "ras", "pbm", "pgm", "ppm",  "png", "jpg",  "jpe", "jpeg"};

    //检查当前拖入的否为url，暂时只支持以url形式拖入
    if (!event->mimeData()->hasUrls()) {
        emit sigImageDragIn(false);
        return;
    }

    //极端情况下，拖拽文件时list有可能为空，不做异常处理时，会闪退。
    QList<QUrl> list = event->mimeData()->urls();
    if (list.isEmpty()) {
        //qDebug() << "拖入数量为空";
        //TODO:后续有需求拖入为空可添加判断
        emit sigImageDragIn(false);
        return;
    }

    QUrl url = list.first();
    if (url.isEmpty()) {
        return;
    }
    QString str = url.fileName();
    if (str.isEmpty()) {
        return;
    }
    event->acceptProposedAction();
    //判断是否为图片
    //    if (formatList.contains(QFileInfo(str).suffix().toLower())) {
    //        event->acceptProposedAction();
    //    } else {
    //        event->ignore(); //否则不接受鼠标事件
    //    }
}

void KWebEngineView::dragLeaveEvent(QDragLeaveEvent *event)
{
    Q_UNUSED(event);
    emit sigImageDragIn(false);
}

void KWebEngineView::dropEvent(QDropEvent *event)
{
    emit sigImageDragIn(false);
    QList<QUrl> urls = event->mimeData()->urls();
    //判断是否为空
    if (urls.isEmpty()) {
        return;
    }
    else if(urls.size() > 1){
        emit sigErrorMoreImage();
        return;
    }
    //拿路径，发信号打开图片
    QString path = urls.first().toLocalFile();
    if (path == "") {
        return;
    }
    //判断文件是否存在
    QFile file(path);
    if(!file.exists()){
        return;
    }
    //当前支持的图片格式
    QStringList supportFormats = {"svg", "gif", "ico", "xpm", "j2k", "jp2", "jng", "wbmp", "xbm", "tiff", "tif", "webp", "bmp", "sr", "ras",
        "pbm", "pgm", "ppm", "png", "jpg", "jpe", "jpeg" };
    //判断是否为支持的图片格式
    if(supportFormats.contains(QFileInfo(path).suffix().toLower()))
    {
        emit sigShowImage(path);
        event->acceptProposedAction();
    }
    else //错误格式在前端进行用户提示
    {
        emit sigErrorFormatImage();
        event->ignore();
    }
}
