#include "reader.h"
#include "global/kaglobal.h"

#include <QBuffer>
#include <QUrl>
#include <QMutexLocker>

#include <ukui-log4qt.h>

#define READ_FILE_PATH "/文档/kylin-aiassistant/.read/"

QString getReaderErrorMsg(SpeechResult errorCode)
{
    QString errorMsg;
    switch(errorCode){
        case SPEECH_SESSION_ERROR:
            errorMsg = "Session错误，请重启服务再试。";
            break;
        case SPEECH_NET_ERROR:
            errorMsg = "未检测到网络链接。请确认网络设置后再试。";
            break;
        case SPEECH_SERVER_ERROR:
            errorMsg = "服务错误或者不可用。请切换网络再试。";
            break;
        case SPEECH_UNAUTHORIZED:
            errorMsg = "权限限制，鉴权失败，无权访问。";
            break;
        case SPEECH_REQUEST_TOO_MANY:
            errorMsg = "请求频率过高，请等一会再试。";
            break;
        case SPEECH_REQUEST_FAILED:
            errorMsg = "请求失败。";
            break;
        case SPEECH_TEXT_LENGTH_INVAILD:
            errorMsg = "输入音频长度超限，请重新发送。";
            break;
        case SPEECH_ADIO_SIZE_INVAILD:
            errorMsg = "音频大小超限，请重新尝试";
                break;
        case SPEECH_PARAM_ERROR:
            errorMsg = "鉴权失败，请重新配置KEY。";
            break;
        default:
            errorMsg = "未知错误！";
            break;
    };
    return errorMsg;
}

static void synthesisResultCallback(SynthesisResult result, void* user_data)
{
    Reader *reader = static_cast<Reader*>(user_data);
    if (reader == nullptr) {
        qDebug() << "Invalid callback user data pointer.";
        return;
    }
    if (reader->m_soundFlag){
        return;
    }
    int errorCode = result.error_code;
    qDebug() << "ssssssss" << errorCode;
    //char *result_text = (char *)result.audio_data;
    //qDebug() << "ssssssss" << result_text;
    if(errorCode !=0)
    {
        qDebug() << "当前错误码为" << errorCode;
        QString errorMsg = getReaderErrorMsg((SpeechResult)errorCode);
        emit reader->reply(errorMsg,true,0,errorCode);
        emit reader->sigReadEnd(0);
        isSpeechSynthesizing = false;
        emit reader->sigChangeReadLoading(isSpeechSynthesizing);
        return;
    }
//    qDebug() << "synthesisResultCallback";
//    qDebug() << "---audio_data---" << result.audio_data;
//    qDebug() << "---audio_rate---" << result.audio_rate;
//    qDebug() << "---audio_format---" << result.audio_format;
//    qDebug() << "---audio_data_length---" << result.audio_data_length;
//    qDebug() << "---userData---" << user_data;

    if (reader->m_file && !reader->m_file->isWritable()) {
        qDebug() << "File is not writable.";
        return;
    }

    // 将音频数据追加到m_file
    if (reader->m_file) {
        reader->m_file->write((const char *)result.audio_data, result.audio_data_length);
    }
     return;
}

static void synthesizedCallback(void* user_data)
{
    Reader *reader = (Reader*)user_data;
    if (reader == nullptr) {
        qDebug() << "Invalid callback user data pointer";
        return;
    }
    if (reader->m_soundFlag) {
        return;
    }
    qDebug() << "synthesizedCallback" << user_data;

    reader->m_file->close();
    reader->startReading();
    return;
}

//static void realTimeSpeechConversionCallback(SynthesisResult result, void *user_data)
//{
//    Reader *reader = (Reader *)user_data;
//    if(reader == nullptr)
//    {
//        qDebug() << "synthesisResultCallback: Invalid callback user data pointer.";
//        return;
//    }

//    if(reader->m_soundFlag)
//    {
//        qDebug() << "result:    output: audio_format:" << result.audio_format << "audio_channel = " << result.audio_channel << "audio_rate = " << result.audio_rate;
//        reader->soundApplay((const char *)result.audio_data, result.audio_data_length);
//        return;
//    }
//}

Reader::Reader(QObject *parent) : QObject(parent)
{
    m_file = new QFile(this);
    m_player = new QMediaPlayer(this);
    m_player->setVolume(80); //0~100音量范围
    initConnect();
//    audio = new AudioPlayer();
}

Reader::~Reader()
{
    m_file->close();
//    if(audio)
//        delete audio;
}

void Reader::startReading()
{
    qDebug() << "reader startReading";
    isSpeechSynthesizing = false;
    emit sigChangeReadLoading(isSpeechSynthesizing);

    if(this->session == nullptr){
        if(!initSession())
        {
            emit sigReadEnd(0);
            isContinuousReadInUse = false;
            return;
        }

    }
    if (!m_file->open(QIODevice::ReadOnly)) {
        qDebug() << "无法打开文件：" <<m_file->fileName();
        return;
    }

    m_player->play();
}

void Reader::pauseReading(int index)
{
    m_player->pause();
    isContinuousReadInUse = false;
}

void Reader::continueReading(int index)
{
    m_player->play();
    isContinuousReadInUse = true;
}

//void Reader::soundApplay(const char *data, int data_size)
//{

//    qDebug() << "音频输出:  data_size = " << data_size;
//    audio->enqueueAudio(QByteArray(data, data_size));
//    audio->m_textSoundFlag = m_textSoundFlag;
//}

////实时声音调用部分
//void Reader::textToSoundOutput(const QString &message, bool is_end, int index)
//{
//    SpeechResult result;
//    m_soundFlag = true;
//    if(m_textSoundFlag)
//    {
//        QJsonObject jsonObject;
//        // jsonObject["voice_name"] = 1;
//        jsonObject["speed"] = 5;
//        jsonObject["volume"] = 5;
//        // jsonObject["pitch"] = 1;
//        const char *params = QJsonDocument(jsonObject).toJson(QJsonDocument::Indented);
//        result = speech_start_continuous_synthesis(this->soundSession, params);
//        if(result != SPEECH_SUCCESS)
//        {
//            qDebug() << "speech_start_continuous_synthesis error";
//        }
//        m_textSoundFlag = false;
//    }

//    QString modifedMsg = message;
//    modifedMsg.replace("\n", "");

//    QByteArray array = modifedMsg.toUtf8();
//    const char *text = array.data();

//    result = speech_continuous_synthesize_write_text_async(this->soundSession, text);
//    if(result != SPEECH_SUCCESS)
//    {
//        qDebug() << "speech_continuous_synthesize_write_text_async error";
//    }

//    if(is_end)
//    {
//        result = speech_stop_continuous_synthesis(this->soundSession);
//        if(result != SPEECH_SUCCESS)
//        {
//            qDebug() << "speech_stop_continuous_synthesis error";
//        }
//        m_textSoundFlag = true;
//    }

//    qDebug() << "文转音频调用：" << m_textSoundFlag << is_end << m_soundFlag;
//}


void Reader::sendText(const QString &msg, int index)
{
    if(!initSession()){
        qDebug() << "初始化session失败，文字发送失败，已经返回";
        emit sigReadEnd(index);
        isContinuousReadInUse = false;
        isSpeechSynthesizing = false;
        emit sigChangeReadLoading(isSpeechSynthesizing);
        return;
     }
    isContinuousReadInUse = true;
    isSpeechSynthesizing = true;
    m_soundFlag = false;

    emit sigChangeReadLoading(isSpeechSynthesizing);

    if (m_player->state() == QMediaPlayer::PlayingState) {
        m_player->pause();
    }
    if (m_file->isOpen()) {
         m_file->close();
    }
    // 清空内容
    // m_file->open(QIODevice::WriteOnly | QIODevice::Truncate);
    // m_file->close();

    // 清空其他音频文件
    QString dataDirPath = QDir::homePath() + READ_FILE_PATH;
    QDir readPathDir(dataDirPath);
    QStringList filters;
    filters << "*.mp3";
    QFileInfoList fileList = readPathDir.entryInfoList(filters, QDir::Files);
    foreach(const QFileInfo &fileInfo, fileList) {
        QFile::remove(fileInfo.absoluteFilePath());
    }

    // 打开新文件进行写入
    QString fileName = generateUniqueFileName();
    m_file->setFileName(fileName);
    m_file->open(QIODevice::WriteOnly | QIODevice::Truncate);
    m_player->setMedia(QUrl::fromLocalFile(m_file->fileName()));

    m_index = index;
    QJsonObject jsonObject;
    // jsonObject["voice_name"] = 1;
    jsonObject["speed"] = 5;
    jsonObject["volume"] = 5;
    // jsonObject["pitch"] = 1;
    const char *params = QJsonDocument(jsonObject).toJson(QJsonDocument::Indented);
//    QString modifedMsg = msg;
//    modifedMsg.replace("\n", "");

    QByteArray array = msg.toUtf8();
    const char* text = array.data();

    qDebug() << "11111char_text" << text;

    SpeechResult result = speech_synthesize_once_async(this->session, params, text);
    if (result != SPEECH_SUCCESS){
        qDebug() <<  "speech_synthesize_once_async error";
        QString error = getReaderErrorMsg(result);
        emit reply(error,true,0,result);
        emit sigReadEnd(0);
        isContinuousReadInUse = false;
        isSpeechSynthesizing = false;
        emit sigChangeReadLoading(isSpeechSynthesizing);
        m_file->close();
        return;
    }

    m_file->open(QIODevice::WriteOnly);

}


bool Reader::initSession()
{
    QString error;
    if(this->session != nullptr)
        return true;

    void *session = nullptr;
    SpeechResult ret = speech_create_session(&session);
    if (ret == SPEECH_SUCCESS){
        KyInfo() << "reader create session successful";
    }
    else{
        KyInfo() << "reader create session error:" <<ret;
        error = getReaderErrorMsg(ret);
        emit reply(error,true,0,ret);
        emit sigReadEnd(0);
        return false;
    }

    ret = speech_init_session(session);
    if (ret != SPEECH_SUCCESS){
        KyInfo() << "reader init session error:" << ret;
        error = getReaderErrorMsg(ret);
        emit reply(error,true,0,ret);
        emit sigReadEnd(0);
        return false;
    }

    if(session)
        this->session = session;
    else
         return false;

//    SpeechSession sound_session = nullptr;
//    SpeechResult rets = speech_create_session(&sound_session);
//    if(rets != SPEECH_SUCCESS)
//    {
//        KyInfo() << "speech create session error:" << rets;
//        error = getReaderErrorMsg(rets);
//        emit reply(error, true, 0, rets, "");
//        emit sigReadEnd(0);
//        return false;
//    }

//    rets = speech_init_session(sound_session);
//    if(rets != SPEECH_SUCCESS)
//    {
//        KyInfo() << "speech init session error :" << rets;
//        error = getReaderErrorMsg(rets);
//        emit reply(error, true, 0, rets, "");
//        emit sigReadEnd(0);

//        return false;
//    }

//    this->soundSession = sound_session;
//    speech_set_synthesizing_callback(this->soundSession, realTimeSpeechConversionCallback, this);

    speech_set_synthesizing_callback(this->session, synthesisResultCallback, this);
    speech_set_synthesized_callback(this->session,synthesizedCallback, this);

    return true;
}

void Reader::closeSession()
{
    if(this->session != nullptr)
        speech_destroy_session(this->session);
    this->session = nullptr;
    isContinuousReadInUse = false;
}

void Reader::initConnect()
{
    connect(m_player, &QMediaPlayer::stateChanged, this, &Reader::onPlayerStateChanged);
}

void Reader::onPlayerStateChanged(QMediaPlayer::State state) {

    if (state == QMediaPlayer::StoppedState) {
        emit sigReadEnd(m_index);
        isContinuousReadInUse = false;
    }
}


QString Reader::generateUniqueFileName()
{
    QString timestamp = QDateTime::currentDateTime().toString("yyyyMMdd_hhmmss");
    return QDir::homePath() + READ_FILE_PATH + "/audio_" + timestamp + ".mp3";
}
