#ifndef KAMULTIMODELSETTINGPAGE_H
#define KAMULTIMODELSETTINGPAGE_H

#include <QWidget>
#include <memory>
#include <QJsonObject>
#include <QJsonDocument>
#include <QJsonArray>
#include <QFile>
#include <QDir>

#include "modelcardwidget.h"
#include "kamodelitem.h"
#include <kdialog.h>
#include <kylin-ai/config.h>
#include <kysdk/applications/kpasswordedit.h>
#include <kysdk/applications/kborderlessbutton.h>

using namespace std;
using namespace kdk;

typedef struct {
    ModelTypeId modelTypeId;
    ModelVendorId modelVendorId;
    QString appId;
    QString apiKey;
    QString apiSecret;

    QString appIdContinuous;
    QString apiKeyContinuous;
    QString secretKeyContinuous;

    QString appIdRealTime;
    QString apiKeyRealTime;

} S_Model;

class AccountCollectionDialog : public QDialog
{
    Q_OBJECT

public:
    explicit AccountCollectionDialog(QWidget *parent = nullptr);
    ~AccountCollectionDialog();

    virtual void keyPressEvent(QKeyEvent *event) override;

signals:

public slots:

private slots:
    void onChangeBtnState();

private:
    void initLayout();
    void initConnect();
    void accept() override;

private:
    class Private;
    shared_ptr<Private> d_ptr;
};


class KAConfigModelSettingDialog : public KDialog
{
    Q_OBJECT
public:
    explicit KAConfigModelSettingDialog(S_Model model, QWidget *parent = nullptr);
    explicit KAConfigModelSettingDialog(QWidget *parent = nullptr);
    ~KAConfigModelSettingDialog();

    // 添加模型-初始化模型UI
    void initUIData(const ModelTypeId vid);
    // 编辑模型-加载模型数据
    void loadModelData(S_Model model);

signals:

public slots:

private slots:
    void onPasswordEditChanged(KPasswordEdit *passwordEdit);
    void onSwitchContinuousChanged(bool isVisible);
    void onSwitchRealTimeChanged(bool isVisible);
    void updateLayout(int index);

private:
    // 布局
    void initLayout();
    // 信号槽
    void initConnect();
    // 根据模型厂商改变尺寸
    void adjustDialogSize();
    // 根据所填内容更新按钮状态
    void updateButtonState();
    // 各模型对应名称
    QString modelVendorToString(const ModelTypeId modelTypeId, const ModelVendorId modelVendorId);

    void accept() override;

private:
    class Private;
    shared_ptr<Private> d_ptr;
};

class KAMultiModelSettingPage : public QWidget
{
    Q_OBJECT
    friend class KAConfigModelSettingDialog;
    friend class AccountCollectionDialog;
public:
    explicit KAMultiModelSettingPage(QWidget *parent = nullptr);
    ~KAMultiModelSettingPage();

signals:


public slots:
    void onChangeTheme();
    void onEditModelItem(ModelTypeId modelTypeId, ModelVendorId modelVendorId);
    void onDeleteModelItem(ModelTypeId modelTypeId, ModelVendorId modelVendorId);

protected:
    bool eventFilter(QObject *watched, QEvent *event) override;

private slots:
    void nlpAddModelItem();
    void visionAddModelItem();
    void speechAddModelItem();

private:
    // 将QMap转换为JSON
    QJsonObject mapToJson(const QMap<ModelVendorId, S_Model>& map);
    // 将JSON转换为QMap
    QMap<ModelVendorId, S_Model> jsonToMap(const QJsonObject& obj);
    // 保存数据到JSON文件
    void saveConfigToJson();
    // 从JSON文件中加载数据
    void loadConfigFromJson();

    // 布局
    void initLayout();
    // 信号槽
    void initConnect();
    // 初始化部署策略
    void initDeployPolicy();

    // 初始化云端页面
    void initCloudModelPage();
    // 初始化私有化页面
    void initPrivateModelPage();
    // 初始化本地页面
    void initLocalModelPage();

    // 显示云端
    void displayCloudModel();
    // 显示私有化
    void displyPrivateModel();
    // 显示本地
    void displyLocalModel();

    // 获取试用帐号
    void getTrialAccount();
    // 添加模型
    void addModelItem(ModelTypeId modelTypeId, ModelVendorId modelVendorId, bool flag);
    // 删除模型
    bool deleteModelItem(ModelTypeId modelTypeId, ModelVendorId modelVendorId);

    // 切换到云端
    void switchCloudModel();
    // 切换到本地
    void switchLocalModel();

    // 切换部署策略
    void switchDeploySelection(ModelCardWidget *clickedModel);
    // 重置部署卡片颜色
    void resetDeployCardColor();
    // 高亮对应部署卡片
    void highlightClickedDeploy(ModelCardWidget *baseModel, ModelCardWidget *highlightModel);

    // 切换已选择模型
    void switchSelectCloudModel(ModelTypeId modelTypeId, ModelVendorId modelVendorId, bool flag);
    // 编辑已选择模型
    void editSelectCloudModel(ModelTypeId modelTypeId, ModelVendorId modelVendorId, S_Model editModel, QWidget *parent);

    // 设置对应模型key值
    void setSelectCloudModel(ModelTypeId modelTypeId, ModelVendorId modelVendorId, bool flag);
    // 恢复模型key默认值
    void resetSelectCloudModel(ModelTypeId modelTypeId, ModelVendorId modelVendorId);

    // 更新ModelMap数据
    void updateModelMap(S_Model editModel, ModelTypeId modelTypeId, ModelVendorId modelVendorId);
    // 检查该模型是否已选择
    bool checkModelSelection(ModelTypeId modelTypeId, ModelVendorId modelVendorId);

private:
    class Private;
    shared_ptr<Private> d_ptr;
};

#endif // KAMULTIMODELSETTINGPAGE_H

