#include "modelcardwidget.h"
#include "global/kaglobal.h"
#include <QGSettings>
#include <QDebug>

#define THEME_QT_SCHEMA  "org.ukui.style"

ModelCardWidget::ModelCardWidget(const QString &modelName, const QIcon &modelImage, bool defaultState, QWidget *parent)
    : QWidget(parent), m_modelName(modelName), m_modelIcon(modelImage),m_state(defaultState)
{
    //this->setFixedSize(118, 64);
    this->setFixedSize(160, 64);
    this->setAttribute(Qt::WA_DeleteOnClose);
    this->setObjectName("modelCard");

    QHBoxLayout *iconButtonLayout = new QHBoxLayout;
    m_iconLabel = new QLabel(this);
    m_iconLabel->setPixmap(m_modelIcon.pixmap(m_modelIcon.actualSize(QSize(24, 24))));

    QIcon selectIcon = QIcon::fromTheme("ukui-selected");
    m_selectIconLabel = new QLabel(this);
    m_selectIconLabel->setPixmap(selectIcon.pixmap(selectIcon.actualSize(QSize(16, 16))));
    const QByteArray id(THEME_QT_SCHEMA);
    QGSettings *mQtSettings = new QGSettings(id, QByteArray(), this);
    connect(mQtSettings, &QGSettings::changed, this, [=](QString key) {
        if (key == "iconThemeName")
            m_selectIconLabel->setPixmap(selectIcon.pixmap(selectIcon.actualSize(QSize(16, 16))));
    });

    m_selectIconLabel->setVisible(false);

    iconButtonLayout->addWidget(m_iconLabel);
    iconButtonLayout->addStretch();

    m_nameLabel = new QLabel(m_modelName, this);
    m_nameLabel->setAlignment(Qt::AlignLeft);

    if (m_state) {
        // 默认状态
        m_selectIconLabel->setVisible(false);
        if (globals::themeColor) {
            QPalette textPalette = this->palette();
            textPalette.setColor(QPalette::WindowText, Qt::white);
            m_nameLabel->setPalette(textPalette);

        } else {
            QPalette textPalette = this->palette();
            textPalette.setColor(QPalette::WindowText, Qt::black);
            m_nameLabel->setPalette(textPalette);
        }
    } else {
        // 选中状态，文字高亮
        QPalette textPalette = this->palette();
        textPalette.setColor(QPalette::WindowText, QColor(qRgb(55, 144, 250)));
        m_nameLabel->setPalette(textPalette);
        m_selectIconLabel->setVisible(true);
    }

    m_vLeftLayout = new QVBoxLayout();
    m_vLeftLayout->addLayout(iconButtonLayout);
    m_vLeftLayout->addSpacing(4);
    m_vLeftLayout->addWidget(m_nameLabel);
    m_vLeftLayout->addStretch();
    m_vLeftLayout->setSpacing(0);
    m_vLeftLayout->setContentsMargins(16,8,0,8);
    m_leftWidget = new QWidget();
    m_leftWidget->setFixedSize(120, 64);
    m_leftWidget->setLayout(m_vLeftLayout);

    m_vRightLayout = new QVBoxLayout();
    m_vRightLayout->addWidget(m_selectIconLabel);
    m_vRightLayout->setAlignment(Qt::AlignRight | Qt::AlignTop);
    m_vRightLayout->setContentsMargins(0,4,4,0);
    m_vRightLayout->addStretch();
    m_rightWidget = new QWidget();
    m_rightWidget->setFixedSize(40, 64);
    m_rightWidget->setLayout(m_vRightLayout);

    m_hLayout = new QHBoxLayout();
    m_hLayout->setContentsMargins(0,0,0,0);
    m_hLayout->addWidget(m_leftWidget);
    m_hLayout->addWidget(m_rightWidget);
    this->setLayout(m_hLayout);

    initConnect();
    installEventFilter(this);
}

ModelCardWidget::~ModelCardWidget()
{

}

void ModelCardWidget::initConnect()
{

}

void ModelCardWidget::changeTheme()
{
    if (globals::themeColor) {
        QPalette pal(this->palette());
        pal.setColor(QPalette::Window, QColor(qRgb(35,36,37)));
        this->setAutoFillBackground(true);
        this->setPalette(pal);
    } else {
        QPalette pal(this->palette());
        //pal.setColor(QPalette::Window, QColor(qRgb(245,245,245)));
        pal.setColor(QPalette::Window, QColor(qRgb(255,255,255)));
        this->setAutoFillBackground(true);
        this->setPalette(pal);
    }
}

void ModelCardWidget::slotLabelSetFontSize(int size)
{
    // 默认大小12px,换算成pt为9
    double lableBaseFontSize = PX_16 * 9/12; // 魔鬼数字，自行处理
    double nowFontSize = lableBaseFontSize * double(size) / 11; // 11为系统默认大小，魔鬼数字，自行处理
    QFont font;
    font.setPointSizeF(nowFontSize);
    m_nameLabel->setFont(font);

}

bool ModelCardWidget::eventFilter(QObject *watched, QEvent *event)
{
    Q_UNUSED(watched);

    if (watched == this) {

        if (event->type() == QEvent::MouseButtonRelease) {
            QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
            if (mouseEvent->button() == Qt::LeftButton) {
                emit clickCard();
            }

            if (globals::themeColor) {
                QPalette pal(this->palette());
                pal.setColor(QPalette::Background, QColor(qRgb(35,36,37)));
                this->setAutoFillBackground(true);
                this->setPalette(pal);
            } else {
                QPalette pal(this->palette());
                //pal.setColor(QPalette::Background, QColor(qRgb(245,245,245)));
                pal.setColor(QPalette::Background, QColor(qRgb(255,255,255)));
                this->setAutoFillBackground(true);
                this->setPalette(pal);
            }

        }
        else if (event->type() == QEvent::MouseButtonPress) {
            QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
            if (mouseEvent->button() == Qt::LeftButton) {
                if (globals::themeColor) {
                    QPalette pal(this->palette());
                    pal.setColor(QPalette::Background, QColor(qRgb(65,65,65)));
                    this->setAutoFillBackground(true);
                    this->setPalette(pal);
                } else {
                    QPalette pal(this->palette());
                    pal.setColor(QPalette::Background, QColor(qRgb(218,218,218)));
                    this->setAutoFillBackground(true);
                    this->setPalette(pal);
                }
            }
        } else if (event->type() == QEvent::HoverEnter || event->type() == QEvent::Enter) {
            if (globals::themeColor) {
                QPalette pal(this->palette());
                pal.setColor(QPalette::Background, QColor(qRgb(46,46,46)));
                this->setAutoFillBackground(true);
                this->setPalette(pal);
            } else {
                QPalette pal(this->palette());
                pal.setColor(QPalette::Background, QColor(qRgb(239,239,239)));
                this->setAutoFillBackground(true);
                this->setPalette(pal);
            }
        }
        else if (event->type() == QEvent::HoverLeave || event->type() == QEvent::Leave) {
            if (globals::themeColor) {
                QPalette pal(this->palette());
                pal.setColor(QPalette::Background, QColor(qRgb(35,36,37)));
                this->setAutoFillBackground(true);
                this->setPalette(pal);
            } else {
                QPalette pal(this->palette());
                //pal.setColor(QPalette::Background, QColor(qRgb(245,245,245)));
                pal.setColor(QPalette::Background, QColor(qRgb(255,255,255)));
                this->setAutoFillBackground(true);
                this->setPalette(pal);
            }
        }
        else if (event->type() == QEvent::Paint) {
            if (m_state) {
                // 默认状态
                globals::borderRound(this,8,8);
                if (globals::themeColor) {
                    QPalette textPalette = this->palette();
                    textPalette.setColor(QPalette::WindowText, Qt::white);
                    m_nameLabel->setPalette(textPalette);
                } else {
                    QPalette textPalette = this->palette();
                    textPalette.setColor(QPalette::WindowText, Qt::black);
                    m_nameLabel->setPalette(textPalette);
                }
            } else {
                // 选中状态
                globals::borderColor(this, 8, 8, QColor(55, 144, 250));
            }
        }
    }
    return QWidget::eventFilter(watched,event);
}
