#include "clipboard.h"

#include <QClipboard>
#include <QApplication>
#include <QDebug>
#include <QFile>
#include "global/kaglobal.h"

Clipboard::Clipboard(QObject *parent) : QObject(parent)
{
    QClipboard *selectClip = QApplication::clipboard();
    connect(selectClip, &QClipboard::selectionChanged, [selectClip, this]() {
        clipText = selectClip->text(QClipboard::Mode::Selection);
        //判断是否为装机模式,装机模式下不支持划词悬浮条
        if(isInstallEquipment()){
            return;
        }
        if(!scribeWordsDisabled
                && !clipText.trimmed().isEmpty()
                && g_gsettings->get("is-stroke-word").toBool())
            {
            emit selectWords();
            }
    });
}

QString Clipboard::getClipText()
{
    return clipText;
}

bool Clipboard::isScribeWordsDisabled()
{
    return scribeWordsDisabled;
}

void Clipboard::clearClipText()
{
    clipText = "";
}

void Clipboard::onScribeWordsDisabled()
{
    scribeWordsDisabled = true;
}

void Clipboard::onScribeWordsIsabled()
{
    scribeWordsDisabled = false;
}

bool Clipboard::isScribeWordsVisible()
{
    if(!scribeWordsDisabled
        && !clipText.trimmed().isEmpty()
        && g_gsettings->get("is-stroke-word").toBool())
    {
       return true;
    }

    return false;
}
//判断是否为装机模式,如果装机文件存在，返回true
bool Clipboard::isInstallEquipment(){
    QString fileName = "/usr/bin/kylin-os-installer-live";
    QFile file(fileName);
    bool flag = file.exists();
    qDebug()<< "判断是否为装机模式文件是否存在flag" << flag;
    return flag;
}
